@@ -2,7 +2,7 @@ package Net::DNS::RR::RT;
 
 use strict;
 use warnings;
-our $VERSION = (qw$Id: RT.pm 1896 2023-01-30 12:59:25Z willem $)[2];
+our $VERSION = (qw$Id: RT.pm 1972 2024-04-21 08:13:19Z willem $)[2];
 
 use base qw(Net::DNS::RR);
 
@@ -19,8 +19,7 @@ use Net::DNS::DomainName;
 
 
 sub _decode_rdata {			## decode rdata from wire-format octet string
-	my ( $self, @argument ) = @_;
-	my ( $data, $offset, @opaque ) = @argument;
+	my ( $self, $data, $offset, @opaque ) = @_;
 
 	$self->{preference}   = unpack( "\@$offset n", $$data );
 	$self->{intermediate} = Net::DNS::DomainName2535->decode( $data, $offset + 2, @opaque );
@@ -29,8 +28,7 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 
 
 sub _encode_rdata {			## encode rdata as wire-format octet string
-	my ( $self,   @argument ) = @_;
-	my ( $offset, @opaque )	  = @argument;
+	my ( $self, $offset, @opaque ) = @_;
 
 	return pack 'n a*', $self->preference, $self->{intermediate}->encode( $offset + 2, @opaque );
 }
@@ -147,6 +145,6 @@ DEALINGS IN THE SOFTWARE.
 =head1 SEE ALSO
 
 L<perl> L<Net::DNS> L<Net::DNS::RR>
-L<RFC1183(3.3)|https://tools.ietf.org/html/rfc1183>
+L<RFC1183(3.3)|https://iana.org/go/rfc1183#section-3.3>
 
 =cut
